import { GlTabs } from '@gitlab/ui';
import VulnerabilityReportLayout from 'ee/security_dashboard/components/shared/vulnerability_report_layout.vue';
import { shallowMountExtended } from 'helpers/vue_test_utils_helper';
import { DASHBOARD_TYPES } from 'ee/security_dashboard/store/constants';

describe('Vulnerability Report Layout component', () => {
  let wrapper;
  const SMALLER_SECTION_CLASS = 'col-xl-7';

  const DummyComponent = {
    name: 'dummy-component',
    template: '<p>dummy component</p>',
  };

  const createWrapper = ({ slots, provide } = {}) => {
    wrapper = shallowMountExtended(VulnerabilityReportLayout, {
      slots,
      provide: {
        dashboardType: DASHBOARD_TYPES.PROJECT,
        glFeatures: { operationalVulnerabilities: false },
        ...provide,
      },
    });
  };

  const findArticle = () => wrapper.find('article');
  const findHeader = () => wrapper.find('header');
  const findStickySection = () => wrapper.findByTestId('sticky-section');
  const findSummarySection = () => wrapper.findByTestId('summary-section');
  const findTabs = () => wrapper.find(GlTabs);

  afterEach(() => {
    wrapper.destroy();
  });

  describe('template slots', () => {
    it('should not render any slot contents by default', () => {
      createWrapper();

      expect(findHeader().exists()).toBe(false);
      expect(findStickySection().exists()).toBe(false);
      expect(findSummarySection().exists()).toBe(false);
      expect(findArticle().classes()).not.toContain(SMALLER_SECTION_CLASS);
    });

    it.each`
      slotName     | findFn
      ${'default'} | ${findArticle}
      ${'header'}  | ${findHeader}
      ${'sticky'}  | ${findStickySection}
      ${'summary'} | ${findSummarySection}
    `('should render the template contents in the correct slot', ({ slotName, findFn }) => {
      createWrapper({
        slots: {
          [slotName]: DummyComponent,
        },
      });

      expect(findFn().find(DummyComponent).exists()).toBe(true);
    });
  });

  describe('with the "operationalVulnerabilities" feature flag on', () => {
    beforeEach(() => {
      createWrapper({
        slots: {
          default: DummyComponent,
          header: DummyComponent,
          sticky: DummyComponent,
          summary: DummyComponent,
        },
        provide: {
          glFeatures: { operationalVulnerabilities: true },
        },
      });
    });

    it.each`
      element              | findFn                | exists
      ${'main'}            | ${findArticle}        | ${true}
      ${'header'}          | ${findHeader}         | ${true}
      ${'sticky section'}  | ${findStickySection}  | ${true}
      ${'summary section'} | ${findSummarySection} | ${true}
    `('should find that $element exists is $exists', ({ exists, findFn }) => {
      expect(findFn().find(DummyComponent).exists()).toBe(exists);
    });
  });

  describe('tabs', () => {
    const createOptions = (dashboardType, operationalVulnerabilities) => ({
      provide: { dashboardType, glFeatures: { operationalVulnerabilities } },
    });

    it.each`
      test                                                                                  | wrapperOptions                                    | exists
      ${'should not find the tabs for the pipeline-level report with the feature flag off'} | ${createOptions(DASHBOARD_TYPES.PIPELINE, false)} | ${false}
      ${'should not find the tabs for the pipeline-level report with the feature flag on'}  | ${createOptions(DASHBOARD_TYPES.PIPELINE, true)}  | ${false}
      ${'should not find the tabs for the project-level report with the feature flag off'}  | ${createOptions(DASHBOARD_TYPES.PROJECT, false)}  | ${false}
      ${'should find the tabs for the project-level report with the feature flag on'}       | ${createOptions(DASHBOARD_TYPES.PROJECT, true)}   | ${true}
      ${'should not find the tabs for the group-level report with the feature flag off'}    | ${createOptions(DASHBOARD_TYPES.GROUP, false)}    | ${false}
      ${'should find the tabs for the group-level report with the feature flag on'}         | ${createOptions(DASHBOARD_TYPES.GROUP, true)}     | ${true}
      ${'should not find the tabs for the instance-level report with the feature flag off'} | ${createOptions(DASHBOARD_TYPES.INSTANCE, false)} | ${false}
      ${'should find the tabs for the instance-level report with the feature flag on'}      | ${createOptions(DASHBOARD_TYPES.INSTANCE, true)}  | ${true}
    `('$test', ({ exists, wrapperOptions }) => {
      createWrapper(wrapperOptions);
      expect(findTabs().exists()).toBe(exists);
    });
  });
});
